# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Draft aware Record Links Schema."""

from invenio_records_resources.resources.records.schemas_links import ItemLink
from marshmallow import Schema


class RecordLinksSchema(Schema):
    """Schema for a record's links."""

    self = ItemLink(template="/api/records/{pid_value}")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 Northwestern University.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

import pytest


def _create_and_publish(service, input_data, identity_simple):
    """Creates a draft and publishes it."""
    draft = service.create(identity_simple, input_data)
    record = service.publish(draft.id, identity_simple)

    assert record.id == draft.id
    assert record._record.revision_id == 1

    return record


# NOTE: We override the function-scoped input_data fixture by a module-scoped
#       one, so it can be used in the module-scoped fixture below.
#       (changing the function-scoped one breaks existing tests)
@pytest.fixture(scope="module")
def input_data():
    """Input data (as coming from the view layer)."""
    return {
        'metadata': {
            'title': 'Test'
        },
    }


@pytest.fixture(scope="module")
def two_versions(app, identity_simple, input_data, service):
    """Create 2 versions."""
    # NOTE: We use app fixture to create/teardown the index only once for
    #       this module.
    record = _create_and_publish(service, input_data, identity_simple)
    recid = record.id
    draft = service.new_version(recid, identity_simple)
    record_2 = service.publish(draft.id, identity_simple)
    service.record_cls.index.refresh()
    return [record, record_2]


def test_search_defaults_to_latest_version(
        service, identity_simple, two_versions):
    results = service.search(identity_simple)

    v1, v2 = two_versions
    assert set([v2.id]) == set([r["id"] for r in results])


def test_search_all_versions(service, identity_simple, two_versions):
    results = service.search(identity_simple, params={"allversions": True})

    v1, v2 = two_versions
    assert set([v1.id, v2.id]) == set([r["id"] for r in results])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2021 CERN.
# Copyright (C) 2020-2021 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Custom service results."""

from flask import current_app
from invenio_records_resources.services.records.results import RecordList
from marshmallow_utils.links import LinksFactory


def _current_host():
    """Function used to provide the current hostname to the link store."""
    if current_app:
        return current_app.config['SITE_HOSTNAME']
    return None


class VersionsList(RecordList):
    """A specialized Result List class made for search versions links.

    The sole reason this exists is to be able to pass the pid_value
    to the dict that is used to generate links.
    """

    def __init__(self, service, identity, results, params,
                 links_config=None, pid_value=None):
        """Overridden RecordList Constructor.

        :params service: a service instance
        :params identity: an identity that performed the service request
        :params results: the search results
        :params params: dictionary of the query parameters
        :params links_config: a links store config
        :params pid_value: pid_value of record
        """
        super().__init__(
            service,
            identity,
            results,
            params,
            links_config=links_config
        )
        self._pid_value = pid_value

    @property
    def links(self):
        """Get the search result links."""
        links = LinksFactory(host=_current_host, config=self._links_config)
        schema = self._service.schema_search_links

        # TODO: Implements a gradual refactoring of schema dumping
        data = schema.dump(
            # It ain't pretty but it will do
            {
                **self._params,
                "_pagination": self.pagination,
                "pid_value": self._pid_value
            },
            context=dict(
                identity=self._identity,
                links_factory=links,
                links_namespace="search",
            )
        )

        return data.get("links")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs."""

import hashlib

from flask import g, request
from flask_resources import CollectionResource
from flask_resources.context import resource_requestctx
from invenio_records_resources.config import ConfigLoaderMixin
from invenio_records_resources.resources import \
    RecordResource as _RecordResource

from invenio_drafts_resources.services.records import RecordDraftService

from .config import RecordResourceConfig, RecordVersionsResourceConfig
from .errors import api_redirect


class RecordResource(_RecordResource):
    """Draft-aware RecordResource."""

    default_config = RecordResourceConfig

    def create(self):
        """Create an item."""
        data = resource_requestctx.request_content
        item = self.service.create(
            g.identity, data, links_config=self.config.draft_links_config)
        return item.to_dict(), 201


class RecordVersionsResource(CollectionResource, ConfigLoaderMixin):
    """Record versions resource."""

    default_config = RecordVersionsResourceConfig

    def __init__(self, service=None, config=None):
        """Constructor."""
        super(RecordVersionsResource, self).__init__(
            config=self.load_config(config))
        self.service = service or RecordDraftService()

    def _get_es_preference(self):
        user_agent = request.headers.get('User-Agent', '')
        ip = request.remote_addr
        user_hash = f"{ip}-{user_agent}".encode('utf8')
        alg = hashlib.md5()
        alg.update(user_hash)
        return alg.hexdigest()

    def search(self):
        """Perform a search over the record's versions.

        GET /records/:pid_value/versions
        """
        identity = g.identity
        hits = self.service.search_versions(
            resource_requestctx.route["pid_value"],
            identity=identity,
            params=resource_requestctx.url_args,
            links_config=self.config.links_config,
            es_preference=self._get_es_preference()
        )
        return hits.to_dict(), 200

    def create(self):
        """Create a new version.

        POST /records/:pid_value/versions
        """
        item = self.service.new_version(
            resource_requestctx.route["pid_value"],
            g.identity,
            links_config=self.config.draft_links_config
        )
        return item.to_dict(), 201

    def read(self):
        """Redirect to latest record.

        GET /records/:pid_value/versions/latest
        """
        item = self.service.read_latest(
            resource_requestctx.route["pid_value"],
            g.identity,
            links_config=self.config.links_config
        )
        api_redirect(location=item["links"]["self"])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Draft aware Record Resource Config override."""

from flask_resources.parsers import HeadersParser, URLArgsParser
from invenio_records_resources.resources import \
    RecordResourceConfig as _RecordResourceConfig
from invenio_records_resources.resources.records.schemas_links import \
    ItemLinksSchema, SearchLinksSchema

from ..drafts.schemas_links import DraftLinksSchema
from .schemas_url_args import SearchURLArgsSchema


class RecordResourceConfig(_RecordResourceConfig):
    """Draft aware Record resource config."""

    draft_links_config = {
        "record": DraftLinksSchema
    }

    request_url_args_parser = {
        "search": URLArgsParser(SearchURLArgsSchema)
    }


class RecordVersionsResourceConfig(RecordResourceConfig):
    """Record resource version config."""

    list_route = "/records/<pid_value>/versions"

    item_route = "/records/<pid_value>/versions/latest"

    links_config = {
        "record": ItemLinksSchema.create(template='/api/records/{pid_value}'),
        # TODO: fix search links schema to pass the <pid_value>
        "search":  SearchLinksSchema.create(
            template='/api/records/{pid_value}/versions{?params*}')
    }

    request_headers_parser = {
        "search": HeadersParser(None),
    }
